# =============================================================================
# URL: https://neutronst4r.xyz
# Filename: autoload/airline/themes/neutronst4r.vim
# Author: neutronst4r
# Email: c7420@posteo.net
# License: GPLv3
# =============================================================================

#!/bin/python

import re
from math import sqrt

black     = '#1a1b26'
red       = '#f7768e'
green     = '#23cab5'
yellow    = '#e0af68'
blue      = '#2384d3'
magenta   = '#6616a8'
cyan      = '#bf5900'
white     = '#888fae'

black_b   = '#313348'
red_b     = '#e43e7f'
green_b   = '#00fff7'
yellow_b  = '#fde559'
blue_b    = '#52b0fd'
magenta_b = '#9617ff'
cyan_b    = '#ff8319'
white_b   = '#c0caf5'

gray1     = '#808080'
gray2     = '#465457'
error     = '#960050'
purewhite = '#000000'
selection = '#00383a'

def writeLine(highlight, guifg=None, guibg=None, guisp=None, gui=None, cterm=None):
    guifg = f"guifg={guifg:<7}" if guifg else " " * 13
    guibg = f"guibg={guibg:<7}" if guibg else " " * 13
    cterm = f"cterm={cterm:<7}" if cterm else " " * 13
    gui   = f"gui={gui:<7}"     if gui   else " " * 11
    guisp = f"guisp={guisp:<7}" if guisp else ""
    if guisp:
        guifg = ""

    line = f"hi {highlight:<15}  {guisp}{guifg}  {guibg}  {gui}  {cterm}".rstrip()
    lines.append(line)

lines = [
    "\" =============================================================================",
    "\" URL: https://neutronst4r.xyz",
    "\" Filename: autoload/airline/themes/neutronst4r.vim",
    "\" Author: neutronst4r",
    "\" Email: c7420@posteo.net",
    "\" License: GPLv3",
    "\" =============================================================================",
    "",
    "let g:colors_name=\"neutronst4r\"",
    "let s:t_Co = exists('&t_Co') && !empty(&t_Co) && &t_Co >= 0 ? &t_Co : -1",
    "set background=dark",
    "",
    "if exists(\"syntax_on\")",
    "    syntax reset",
    "endif",
    "hi clear",
    "",
]
skip = len(lines)

writeLine("Normal",         guifg=white_b,   guibg=black,      cterm="None")
writeLine("Comment",        guifg=white)
writeLine("CursorLine",                      guibg=black_b,    cterm="None")
writeLine("CursorColumn",                    guibg=black_b)
writeLine("LineNr",         guifg=white,     guibg=black_b)
writeLine("NonText",        guifg=white,     guibg=black)
lines.append("")
writeLine("Boolean",        guifg=magenta_b)
writeLine("Character",      guifg=yellow_b)
writeLine("Number",         guifg=magenta_b)
writeLine("String",         guifg=green)
writeLine("Conditional",    guifg=red_b,                       gui="bold")
writeLine("Constant",       guifg=magenta_b,                   gui="bold")
writeLine("Cursor",         guifg=white_b)
#writeLine("Debug",          guifg="#BCA3A3",                  gui="bold")
writeLine("Define",         guifg=yellow)
#writeLine("Delimiter",      guifg="#8F8F8F")
writeLine("DiffAdd",        guifg=green_b,   guibg=black)
writeLine("DiffChange",     guifg=yellow,  guibg=black)
writeLine("DiffDelete",     guifg=error,     guibg=black)
writeLine("DiffText",                        guibg=black,      gui="italic,bold")
lines.append("")
writeLine("Directory",      guifg=green,                       gui="bold")
writeLine("Error",          guifg=error,     guibg=black)
writeLine("ErrorMsg",       guifg=error,     guibg=black,      gui="bold")
writeLine("Exception",      guifg=green,                       gui="bold")
writeLine("Float",          guifg=magenta_b)
writeLine("FoldColumn",     guifg=red_b,     guibg=white_b)
writeLine("Folded",         guifg=red_b,     guibg=white_b)
writeLine("Function",       guifg=green_b)
writeLine("Identifier",     guifg=cyan_b)
writeLine("Ignore",         guifg=gray1,     guibg="bg")
lines.append("")
writeLine("Keyword",        guifg=blue_b,                      gui="bold")
writeLine("Label",          guifg=yellow_b,                    gui="none")
writeLine("Macro",          guifg=yellow,                      gui="italic")
lines.append("")
writeLine("MatchParen",                      guibg=white,      gui="bold")
writeLine("ModeMsg",        guifg=green_b)
writeLine("MoreMsg",        guifg=green_b)
writeLine("Operator",       guifg=red_b)
lines.append("")
lines.append("\" complete menu")

# TODO PMENU
# https://www.youtube.com/watch?v=Ey6Gs_QApzk
writeLine("Pmenu",          guifg=green_b,   guibg=white_b)
writeLine("PmenuSel",                        guibg=gray1)
writeLine("PmenuSbar",                       guibg=black)
writeLine("PmenuThumb",     guifg=green_b)

lines.append("")
writeLine("PreCondit",      guifg=green,                       gui="bold")
writeLine("PreProc",        guifg=green)
writeLine("Question",       guifg=green_b)
writeLine("Repeat",         guifg=red_b,                       gui="bold")
writeLine("Search ",        guifg=black,     guibg=green)
writeLine("CurSearch ",     guifg=black,     guibg=red)
writeLine("IncSearch",      guifg=green,     guibg=white_b)
lines.append("")
lines.append("\" marks column")
writeLine("SignColumn",     guifg=white,     guibg=selection)
writeLine("SpecialChar",    guifg=red_b,                       gui="bold")
writeLine("SpecialComment", guifg=green,                       gui="bold")
writeLine("Special",        guifg=cyan,      guibg="bg",       gui="italic")
writeLine("SpecialKey",     guifg=yellow,                      gui="italic")
lines.append("if has(\"spell\")")
writeLine("    SpellBad",   guisp=error,                       gui="undercurl")
writeLine("    SpellCap",   guisp=magenta_b,                   gui="undercurl")
writeLine("    SpellLocal", guisp=green_b,                     gui="undercurl")
writeLine("    SpellRare",  guisp=white_b,                     gui="undercurl")
lines.append("endif")
writeLine("Statement",      guifg=red_b,                       gui="bold")
writeLine("StatusLine",     guifg=white_b,   guibg="bg")
writeLine("StatusLineNC",   guifg=gray1,     guibg="bg")
writeLine("StorageClass",   guifg=cyan_b,                      gui="italic")
writeLine("Structure",      guifg=blue)
writeLine("Tag",            guifg=red_b,                       gui="italic")
writeLine("Title",          guifg=red)
writeLine("Todo",           guifg=white_b,   guibg="bg",       gui="bold")
lines.append("")
writeLine("Typedef",        guifg=blue)
writeLine("Type",           guifg=blue,                        gui="none")
writeLine("Underlined",     guifg=gray1,                       gui="underline")
lines.append("")
writeLine("VertSplit",      guifg=white_b,   guibg=black,      gui="bold")
writeLine("VisualNOS",                       guibg=selection)
writeLine("Visual",                          guibg=selection)
writeLine("WarningMsg",     guifg=yellow,                      gui="bold")
writeLine("WildMenu",       guifg=blue,      guibg=white_b)


with open('neutronst4r.vim', 'w') as file:
    file.write("\n".join(lines))

# =====================================
# Part Two: Get closest terminal colors
# =====================================

def get_termcolors():
    with open('termcolors', 'r') as file:
        colors = file.readlines()
    for i, color in enumerate(colors):
        colors[i] = color.split()
        colors[i][0] = int(colors[i][0])
        colors[i][1] = (int(colors[i][1][1:3], base=16),
                        int(colors[i][1][3:5], base=16),
                        int(colors[i][1][5:7], base=16))
    return colors

def closest256(hex):
    if hex:
        if hex[0] == '#':
            R0, G0, B0 = int(hex[1:3], base=16), int(hex[3:5], base=16), int(hex[5:7], base=16)
            distances = []
            for i in termcolors:
                R1, G1, B1 = i[1][0], i[1][1], i[1][2]
                R_mean = (R0 + R1)/2.0
                dR = R0 - R1
                dG = G0 - G1
                dB = B0 - B1
                #print(i[0], R_mean, dR, dG, dB)
                distance = sqrt((2 + R_mean / 256) * dR**2  +  4.0 * dG**2  +  (2 + (255 - R_mean) / 256) * dB**2)
                distances.append(distance)
            minimum = min(distances)
            res = [i for i, j in enumerate(distances) if j == minimum]
            return res[0]
    return hex

def writeLine256(highlight, ctermfg=None, ctermbg=None, cterm=None):
    ctermfg = closest256(ctermfg)
    ctermbg = closest256(ctermbg)
    ctermfg = f"ctermfg={ctermfg:<7}" if ctermfg else " " * 15
    ctermbg = f"ctermbg={ctermbg:<7}" if ctermbg else " " * 15
    cterm   = f"cterm={cterm:<7}" if cterm else " " * 13
    return f"    hi {highlight:<15}  {ctermfg}  {ctermbg}  {cterm}".rstrip()

with open('neutronst4r.vim', 'r') as file:
    lines = file.readlines()
lines = lines[skip:]

newlines = ["", "", "if s:t_Co >= 256"]
termcolors = get_termcolors()
pattern = re.compile(r'hi(\s*\w*)\s*(.*)')
for line in lines:
    m = re.search(pattern, line)
    if m:
        line =f"hi{m.group(1):<17}"

        ctermfg = None
        ctermbg = None
        cterm   = None
        for i in m.group(2).split():
            key, value = i.split('=')
            if key in ['guifg', 'guisp']:
                ctermfg = value
                continue
            if key == 'guibg':
                ctermbg = value
                continue
            if key == 'cterm':
                cterm = value
                continue
        newlines.append(writeLine256(m.group(1), ctermfg=ctermfg, ctermbg=ctermbg, cterm=cterm))
    else:
        newlines.append(("    " + line.replace('\n', '')).rstrip())
newlines.extend(['    unlet s:t_Co', '    finish', 'endif'])

with open('neutronst4r.vim', 'a') as file:
    file.write("\n".join(newlines))


# =====================================
# Part Three: Generate Airline Theme
# =====================================
# foreground.old      = #a9b1d6
# background_alt.old  = #232433
# background_gray.old = #444B6A

lines = [
    "\" =============================================================================",
    "\" URL: https://neutronst4r.xyz",
    "\" Filename: autoload/airline/themes/neutronst4r.vim",
    "\" Author: neutronst4r",
    "\" Email: c7420@posteo.net",
    "\" License: GPLv3",
    "\" =============================================================================",
    "",
    "let g:airline#themes#neutronst4r#palette = {}",
    "",
    f"    let s:foreground      = [ '{white_b}',"   f'{closest256(white_b):>3} ]"',
    f"    let s:background      = [ '{black}',"     f'{closest256(black):>3} ]"',
    f"    let s:background_alt  = [ '{black_b}',"   f'{closest256(black_b):>3} ]"',
    f"    let s:background_grey = [ '{white}',"     f'{closest256(white):>3} ]"',
    f"    let s:red             = [ '{red_b}',"     f'{closest256(red_b):>3} ]"',
    f"    let s:yellow          = [ '{yellow_b}',"  f'{closest256(yellow_b):>3} ]"',
    f"    let s:green           = [ '{green_b}',"   f'{closest256(green_b):>3} ]"',
    f"    let s:blue            = [ '{blue_b}',"    f'{closest256(blue_b):>3} ]"',
    f"    let s:purple          = [ '{magenta_b}'," f'{closest256(magenta_b):>3} ]"',
    f"    let s:orange          = [ '{cyan_b}',"    f'{closest256(cyan_b):>3} ]"',
    "\"                                     guifg            guibg                 ctermfg          ctermbg",
    "let s:airline_mode_normal         = [ s:background[0], s:red[0],             s:background[1], s:red[1]           ]",
    "let s:airline_mode_insert         = [ s:background[0], s:green[0],           s:background[1], s:green[1]          ]",
    "let s:airline_mode_replace        = [ s:background[0], s:orange[0],          s:background[1], s:orange[1]             ]",
    "let s:airline_mode_visual         = [ s:background[0], s:blue[0],            s:background[1], s:blue[1]            ]",
    "let s:airline_mode_inactive       = [ s:foreground[0], s:background_grey[0], s:foreground[1], s:background_grey[1] ]",
    "let s:airline_info                = [ s:foreground[0], s:background_grey[0], s:foreground[1], s:background_grey[1] ]",
    "let s:airline_statusline          = [ s:foreground[0], s:background_alt[0],  s:foreground[1], s:background_alt[1]  ]",
    "",
    "function! s:neutronst4r_color_map(mode)",
    "    return airline#themes#generate_color_map(a:mode, s:airline_info, s:airline_statusline)",
    "endfunction",
    "",
    "let g:airline#themes#neutronst4r#palette.normal = s:neutronst4r_color_map(s:airline_mode_normal)",
    "let g:airline#themes#neutronst4r#palette.insert = s:neutronst4r_color_map(s:airline_mode_insert)",
    "let g:airline#themes#neutronst4r#palette.replace = s:neutronst4r_color_map(s:airline_mode_replace)",
    "let g:airline#themes#neutronst4r#palette.inactive = s:neutronst4r_color_map(s:airline_mode_inactive)",
    "let g:airline#themes#neutronst4r#palette.visual = s:neutronst4r_color_map(s:airline_mode_visual)",
    "let g:airline#themes#neutronst4r#palette.terminal = s:neutronst4r_color_map(s:airline_mode_insert)",
    "let g:airline#themes#neutronst4r#palette.normal.airline_term = s:airline_statusline",
    "let g:airline#themes#neutronst4r#palette.terminal.airline_term = s:airline_statusline",
    "let g:airline#themes#neutronst4r#palette.visual.airline_term = s:airline_statusline",
]

with open('../autoload/airline/themes/neutronst4r.vim', 'w') as file:
    file.write("\n".join(lines))
